//
//  ViewController.swift
//  DKRefImplSwift
//
//  Updated March 2020.
//  Copyright © 2020 The Nielsen Company, All rights reserved.
//
import UIKit
import AVKit
import CoreLocation
import AdSupport
import AVFoundation
import NielsenAppApi

class ViewController: UIViewController , NielsenAppApiDelegate, AVPlayerViewControllerDelegate {
    // public
    public var totalVideosPlayed = 0
    public var totalVideos : Int!
    public var videoType : Int!
    
    // private
    @IBOutlet weak private var MetaInfo: UILabel!
    private var nielsenApi : NielsenAppApi!
    private var player : AVPlayer!
    private var controller : AVPlayerViewController!
    private var sdkMethods : SDKMethods!
    private var data : [String : Any]!
    private var playbackStarted : Bool = false
    private var playbackComplete : Bool = false
    private var isBuffering : Int = 0
    private var lastBufferEvent : String!
    private var isPaused : Bool = false;
    private var lastPlayerRate : Float = 0.0
    private var playheadTimer : Timer!
    private var lastStopTimeForBufferEmpty : Float64! = -1.0
    private var sendMetadataAfterStop : Bool = false
    // constants
    private let playbackLikelyToKeepUp : String = "playbackLikelyToKeepUp"
    private let playbackBufferEmpty : String = "playbackBufferEmpty"
    private let keyPathRate : String = "rate"
    private let keyPathStatus : String = "status"
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.view.backgroundColor = UIColor(patternImage: UIImage(named: "new_ios_bg.png")!)
        
        //Mark: In NielsenInit class we are initialising the NielsenApi.
        
        //Getting the instance of NielsenApi
        self.nielsenApi = NielsenInit.getAppApi()
        //Mark: In SDKMethods class we wrote methods which creates content,Ad,DTVR objects
        sdkMethods = SDKMethods()
        
        createPlayer()
    }
    
    private func createPlayer() {
        // Remove NotificationCenter and readd at the end!
        if player != nil {
            print("Remove NotificationCenter and readd at the end!")
            NotificationCenter.default.removeObserver(self)
        }
        
        //creating player
        playbackStarted = false
        isBuffering = 0 // prevent playheadPosition for Nielsen to be sent before the initial buffering is complete
        lastBufferEvent = ""
        startTimestampLive = -1.0
        startTimeLive = -1.0
        lastPlayheadPos = INT64_MAX
        lastPlayerRate = 0.0
        isPaused = false
        sendMetadataAfterStop = false
        lastStopTimeForBufferEmpty = -1.0
        
        if totalVideosPlayed == 0 {
            //For first time sending Channel info to SDK using "play" method.
            print("==##> Nielsen SDK .play(sdkMethods.loadChannelInfo())")
            nielsenApi.play(sdkMethods.loadChannelInfo())
        }
        
        //Sending metadata to SDK.
        //loading video content metadata data
        self.data = sdkMethods.loadContent(contentType: videoType)
        print("==##> Nielsen SDK nielsenApi.loadMetadata")
        nielsenApi.loadMetadata(self.data)
        MetaInfo.text = "\(self.data!)"  //Displays on screen metadata being passed to NielsenSDK
        
        if videoType == Constants.vodContentWithPrerollAd && totalVideosPlayed == 0 {
            //loading Ad data
            self.data = sdkMethods.loadPreRollAd()
            print("==##> Nielsen SDK nielsenApi.loadMetadata - PrerollAd ")
            nielsenApi.loadMetadata(self.data)
        }
        
        player  = AVPlayer.init(url: sdkMethods.url! as URL)
        controller = AVPlayerViewController()
        controller.view.frame = CGRect(x:0 , y:100, width: self.view.frame.width, height: 300)
        controller.player = player;
        controller.showsPlaybackControls = true;
        controller.delegate = self;
        
        self.addChildViewController(controller)
        self.view.addSubview(controller.view)
        
        //Setting observer to know the completion of video
        addObserver()
    }
    
    private var startTimeLive : Float64! = -1.0
    private var startTimestampLive : Float64 = -1.0
    private var lastPlayheadPos : Int64 = INT64_MAX
    
    @objc private func updatePlayheadPosition() {
        let currentTime : Double = self.player!.currentTime().seconds;
        var pos = Int64(currentTime)
        let duration : Double = self.player!.currentItem!.duration.seconds
        
        if sdkMethods.isLive {
            // For event live stream we keep the EPG planned duration
            // For 24/7 Live stream, duration = 24*60*60 // 1 day in seconds as per Nielsen API documentation
            if  startTimestampLive == -1.0 {
                startTimestampLive = NSDate().timeIntervalSince1970
            }
            if startTimeLive == -1.0 {
                startTimeLive = currentTime
            }
            pos = Int64(currentTime - startTimeLive + startTimestampLive)
        }
        
        //Sending playHeadPosition to Nielsen SDK.
        if self.playbackStarted && self.isPlaying() && lastPlayheadPos != pos {
            lastPlayheadPos = pos
            if sendMetadataAfterStop && !sdkMethods.isAd {
                print("==##> updatePlayheadPosition : Nielsen SDK loadMetadata()")
                nielsenApi.loadMetadata(self.data)
                sendMetadataAfterStop = false
            }
            print("==##> Nielsen SDK updatePlayheadPosition, pos = " + String(pos) + " / " + ( duration.isNaN ? "--" :  String(Int64(duration)) ))
            self.nielsenApi?.playheadPosition(pos)
        }
    }
    
    private func addObserver() {
        let nc = NotificationCenter.default
        nc.addObserver(self, selector: #selector(reactToTermination(_:)), name: Globals.terminateNotification, object: nil)
        if player != nil && player.currentItem != nil {
            //observer fires on completion video.
            nc.addObserver(self, selector: #selector(playerDidFinishPlaying), name: NSNotification.Name.AVPlayerItemDidPlayToEndTime, object: player?.currentItem)
            //Adding observer to player to track play,pause and reverse
            player.addObserver(self, forKeyPath: keyPathRate, options: NSKeyValueObservingOptions.new, context: nil)
            //Adding observer to player to track play,pause and reverse
            player.addObserver(self, forKeyPath: keyPathStatus, options: NSKeyValueObservingOptions.new, context: nil)
            
            player.currentItem?.addObserver(self, forKeyPath: playbackBufferEmpty, options: NSKeyValueObservingOptions.new, context: nil)
            player.currentItem?.addObserver(self, forKeyPath: playbackLikelyToKeepUp, options: NSKeyValueObservingOptions.new, context: nil)
        }
    }
    
    
    private func isPlaying() -> Bool {
        if player != nil && player.rate == 1 && !isPlayerBuffering() {
            return true
        }
        return false
    }
    
    private func isPlayerBuffering() -> Bool {
        if lastBufferEvent == playbackLikelyToKeepUp || isBuffering <= 0 {
            return false
        } else {
            return true
        }
    }
    //rate 0.0 = Video Pause or stopped
    //rate 1.0 = Video played or resumed
    override func observeValue(forKeyPath keyPath: String?, of object: Any?, change: [NSKeyValueChangeKey : Any]?, context: UnsafeMutableRawPointer?) {
        
        if object is AVPlayerItem {
            switch keyPath {
            case playbackBufferEmpty:
                // Buffering Start
                print("==> Player Observer notification " + playbackBufferEmpty)
                isBuffering += 1
                if totalVideosPlayed == 0 && playheadTimer == nil {
                    playheadTimer = Timer.scheduledTimer(timeInterval: 1.0, target: self, selector: #selector(self.updatePlayheadPosition), userInfo: nil, repeats: true)
                }
                let startTime : Double = sdkMethods.isLive ? startTimeLive : 0.0 // 0 for VoD
                let currentPlayerTime : Double = player.currentTime().seconds
                let viewingTime : Double = sdkMethods.isLive ? abs(currentPlayerTime - startTime) : currentPlayerTime
                let ViewingTimeSinceLastStop : Double = lastStopTimeForBufferEmpty == -1.0 ? 0.0 : (NSDate().timeIntervalSince1970 - lastStopTimeForBufferEmpty)
                
                print("==> Player Observer notification " + playbackBufferEmpty + " ViewingTimeSinceLastStop = " + String(ViewingTimeSinceLastStop) + " viewingTime = " + String(viewingTime) )
                // startTimeLive = -1 for Live Stream when uninitialized
                if startTime != -1.0 && viewingTime > 1.0 && ViewingTimeSinceLastStop >= 15.0 && !sendMetadataAfterStop {
                    print("==##>  " + playbackBufferEmpty + "Nielsen SDK stop()")
                    nielsenApi.stop()
                    // sendMetadataAfterStop = true // not for DK
                }
                lastStopTimeForBufferEmpty = NSDate().timeIntervalSince1970
                lastBufferEvent = playbackBufferEmpty
                break
            case playbackLikelyToKeepUp:
                // Buffering End
                isBuffering -= 1
                lastBufferEvent = playbackLikelyToKeepUp
                print("==> Player Observer notification " + "playbackLikelyToKeepUp")
                break
            default:
                break
            }
        } else {
            switch keyPath {
            case keyPathRate:
                let rate = change?[NSKeyValueChangeKey.newKey] as! Float
                handlePlayPause(rate)
                break
            case keyPathStatus:
                print("==> Player status changed")
                if player.status == .readyToPlay {
                    print("==> Player status changed: AVPlayerItemStatus.readyToPlay")
                    player.play()
                }
                break
            default:
                break
            }
        }
    }
    
    override func viewDidDisappear(_ animated: Bool) {
        terminatePlaybackSession()
        player.pause()
        super.viewDidDisappear(animated)
    }
    
    @objc private func reactToTermination(_ sender: Notification) {
        terminatePlaybackSession()
    }
    
    @objc private func playerDidFinishPlaying(note: NSNotification) {
        
        //As 1 video completed playing, incrementing the variable value.
        totalVideosPlayed += 1
        
        if totalVideosPlayed == totalVideos {
            //When total videos finishes, terminate playback session
            terminatePlaybackSession()
        } else {
            print("==##> playerDidFinishPlaying Nielsen SDK stop()")
            nielsenApi.stop()
            if sdkMethods.isAd {
                //On completion of "AD" call "stop" method.
                createPlayer()
            } else {
                // only main content playback complete
                print("==##> End main content playback Nielsen SDK end()")
                nielsenApi.end()
            }
        }
    }
    
    private func handlePlayPause(_ rate : Float) {
        print("==> Player Observer notification for 'rate =' " + String(rate))
        if rate == 0.0 {
            print("Playback paused")
            //On Video pause calling the "stop" method of SDK.
            if !playbackComplete && self.player!.currentTime().seconds > 0 && !isPaused {
                print("==##> Nielsen SDK updatePlayheadPosition - .stop()")
                isPaused = true
                nielsenApi.stop()
                // sendMetadataAfterStop = true // not for DK
            }
        } else if rate == 1.0 {
            print("normal playback")
            if !playbackStarted {
                playbackStarted = true
            } else {
                if isPaused {
                    isPaused = false
                }
                // Correct eventual AVPlayer inconsistent Buffering notifications
                // i.e. following events sequence
                // playbackBufferEmpty, playbackBufferEmpty, playbackLikelyToKeepUp, playbackLikelyToKeepUp, playbackBufferEmpty
                if isBuffering > 0 {
                    isBuffering = 0
                    print("==##> Buffering state corrected since player is playing!")
                }
            }
        }
        lastPlayerRate = rate
    }
    
    private func terminatePlaybackSession() {
        if playheadTimer != nil {
            playbackStarted = false
            playbackComplete = true;
            print("==##> terminatePlaybackSession Nielsen SDK end()")
            nielsenApi.end()
            playheadTimer.invalidate()
            playheadTimer = nil
            controller.view.isUserInteractionEnabled = false
            controller.showsPlaybackControls = false
            print("Remove NotificationCenter terminatePlaybackSession")
            NotificationCenter.default.removeObserver(self)
            nielsenApi = nil
        }
    }
    
    public func nielsenAppApi(_ appApi: NielsenAppApi!, eventOccurred event: [AnyHashable : Any]!) {
        print("==##!!> ViewController -- Nielsen SDK eventOccurred")
    }
    
    public func nielsenAppApi(_ appApi: NielsenAppApi!, errorOccurred error: [AnyHashable : Any]!) {
        print("==##!!> ViewController -- Nielsen SDK errorOccurred")
    }
}
